; #DllStruct# ;==============================================================================
; Title..........: DllStruct UDF
; Filename.......: DllStruct.au3
; Date...........: 31.10.2012
; Description ...: Contains functions to save and delete DllStructs
; Author.........: Mars
; Functions......: _DllStructDeleteByPtr
;                  _DllStructSave
;                  _DllStructSaveSetInterval
; ===========================================================================================

Global $__Save_Array[1], $__Save_Anzahl = 0, $__Save_Intervall = 5

; #FUNCTION# ;===============================================================================
; Name...........: _DllStructSaveSetInterval
; Description ...: Sets resizing interval of the DllStruct saving Array
; Syntax.........: _DllStructSaveSetInterval($i)
; Parameters ....: $iInterval - Integer Value between 1 and 1000
; Return values .: Success - Returns True
;                  Failure - Returns False - The value was not an Integer or not between 1 and 1000
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......: _DllStructSave, _DllStructDeleteByPtr
; Link ..........;
; Example .......; Yes
; ;=========================================================================================
Func _DllStructSaveSetInterval($iInterval)
	If IsInt($iInterval) And $iInterval > 0 And $iInterval <= 1000 Then
		$__Save_Intervall = $iInterval
		Return True
	Else
		Return False
	EndIf
EndFunc   ;==>_DllStructSaveSetInterval

; #FUNCTION# ;===============================================================================
; Name...........: _DllStructSave
; Description ...: Saves a Struct from being deleted automatically when no variable contains it
; Syntax.........: _DllStructSave($vStruct)
; Parameters ....: $vStruct - Struct returned by DllStructCreate
; Return values .: None - This function should always succeed
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......: _DllStructDeleteByPtr
; Link ..........;
; Example .......; Yes
; ;==========================================================================================
Func _DllStructSave($vStruct)
	Local $iUbound = UBound($__Save_Array)
	$__Save_Anzahl += 1
	If $iUbound < $__Save_Anzahl Then
		ReDim $__Save_Array[$iUbound + $__Save_Intervall]
	EndIf
	$__Save_Array[$__Save_Anzahl - 1] = $vStruct
EndFunc   ;==>_DllStructSave

; #FUNCTION# ;===============================================================================
; Name...........: _DllStructDeleteByPtr
; Description ...: Deletes a previously saved DllStruct by Ptr
; Syntax.........: _DllStructDeleteByPtr($iPointer)
; Parameters ....: $iPointer - Ptr returned by DllStructGetPtr
; Return values .: Success - Returns True
;                  Failure - Returns False and sets @error
;                  1 - $iPointer is invalid
;                  2 - The Struct that should be deleted was not found
; Author ........: Mars
; Modified.......:
; Remarks .......:
; Related .......: _DllStructSave
; Link ..........;
; Example .......; Yes
; ;=========================================================================================
Func _DllStructDeleteByPtr($iPointer)
	If Not Ptr($iPointer) Then Return SetError(1, 0, False)
	For $i = 0 To UBound($__Save_Array) - 1 Step 1
		If $iPointer = DllStructGetPtr($__Save_Array[$i]) Then
			$__Save_Anzahl -= 1
			$__Save_Array[$i] = $__Save_Array[$__Save_Anzahl]
			$__Save_Array[$__Save_Anzahl] = 0
			If ($__Save_Anzahl + $__Save_Intervall) < UBound($__Save_Array) Then
				ReDim $__Save_Array[$__Save_Anzahl + 1]
			EndIf
			Return True
		EndIf
	Next
	Return SetError(2, 0, False)
EndFunc   ;==>_DllStructDeleteByPtr